/* Spritelib demo application: "space invaders" style game.
  Orion Sky Lawlor, olawlor@acm.org, 2010-04-05 (Public Domain) */
#include "spritelib/spritelib.h"
#include "spritelib/spritelib.cpp"
#include <list>

float enemyw=150.0,enemyh=75.0;
class enemy {
public:
	float created; /* time of creation (for disposal) */
	float x,y; /* current location onscreen */
	void draw(spritelib &lib) {
		static spritelib_tex enemytex=lib.read_tex("dogmop.jpg");
		lib.draw(enemytex,
			x,y, // position
			enemyw,enemyh, // size
			4*sin(lib.time*10+100*created));
	}
	void physics(spritelib &lib) {
		float speed=-100.0; /* motion pixels/second (positive == down) */
		y+=speed*lib.dt; /* pixels/second * seconds = pixels */
		x+=((rand()%1000)-500)*lib.dt; /* "drunken walk" */
	}
};

class shot {
public:
	float created; /* time of creation (for disposal) */
	float x,y; /* current location onscreen */
	float size; /* onscreen size (pixels) */
	float angle; /* rotation angle (degrees) */
	void draw(spritelib &lib) {
		static spritelib_tex glow=lib.read_tex("glow.jpg");
		lib.draw(glow,
			x,y, // position
			size,size,
			angle,
			true); // glow!
	}
	void physics(spritelib &lib) {
		float speed=800.0; /* motion pixels/second (positive == down) */
		y+=speed*lib.dt; /* pixels/second * seconds = pixels */
		size+=100.0*lib.dt; /* size growth rate */
		angle+=1230*lib.dt; /* angular rotation rate */
	}
	
	// Return true if this shot hits this enemy
	bool hits(const enemy &e) {
		return between(e.x-0.5*enemyw,x,e.x+0.5*enemyw) &&
		    between(e.y-0.5*enemyh,y,e.y+0.5*enemyh);
	}
	// Return true if x is between low and high
	bool between(float low,float x,float high) {
		return (low<x)&&(x<high);
	}
};

/* Convert int to string */
std::string int_to_string(int i) {
	char buf[1000]; sprintf(buf,"%d",i); return buf;
}

void spritelib_draw_screen(spritelib &lib) {
	static float pausetime=0.0; /* total time spent paused */
	if (lib.key[' '] || lib.key[0x1B]) { /* spacebar or escape pauses */
		pausetime+=lib.dt;
		lib.dt=0.0; /* HACK!  stop physics, by setting timestep to zero */
	}
	lib.time-=pausetime; /* remove time spent paused */

/* Background */
	static spritelib_tex background=lib.read_tex("sunset.jpg");
	float ken_burns=1.1-0.1*cos(0.15*lib.time); // slowly zoom in and out
	lib.draw(background,lib.screenw/2.0,lib.screenh/2.0,
		lib.screenw*ken_burns,lib.screenh*ken_burns);
	
/* "points" */
	static int points=0;
	points+=rand()&3;
	float color[4]={1.0,1.0,0.0,1.0}; // RGBA color of text
	lib.text(int_to_string(points),10,50,color);
	
/* Player character */
	static spritelib_tex cat=lib.read_tex("cat_alpha.png");
	float playerX=lib.mousex;
	float playerY=50;
	lib.draw(cat,
		playerX, // position
		playerY,
		100, // size
		100,
		0); // angle cat facing down

/* Enemies */
	static std::list<enemy> enemys;
	static float last_enemy=0.0;
	float enemy_time=1.0/10.0; /* time between new enemies */
	if (lib.time>last_enemy+enemy_time) { /* make new enemy */
		last_enemy=lib.time;
		enemy e;
		e.created=lib.time;
		e.x=rand()%(int)lib.screenw;
		e.y=lib.screenh+enemyh;
		enemys.push_back(e);
	}
	/* Draw all enemies */
	for (std::list<enemy>::iterator it=enemys.begin();it!=enemys.end();++it) 
	{
		enemy &e=(*it);
		e.physics(lib);
		e.draw(lib);
		if (e.y<0.0) { /* enemy made it past top of screen! */
			std::cout<<"GAME OVER!\n";
			exit(0); /* GAME OVER, man! */
		}
	}

/* Onscreen ordinance */
	static std::list<shot> shots; /* <- static, so shots stick around between frames */
	if (lib.mouseleft) { /* spacebar is down--consider adding shot */
		static float last_shot=0.0;
		float shot_time=1.0/12.0; /* time between shots */
		if (lib.time>last_shot+shot_time) { /* limits shot rate */
			last_shot=lib.time;
			shot s; 
			s.created=lib.time;
			s.x=playerX; s.y=playerY+40.0;
			s.size=40.0; /* onscreen size */
			s.angle=rand()%360;
			shots.push_back(s);
		}
	}
	
	/* Draw all shots */
	for (std::list<shot>::iterator it=shots.begin();it!=shots.end();/* see end */) 
	{
		shot &s=(*it);
		s.physics(lib);
		s.draw(lib);
		bool remove_shot=false;
		if (s.created>2.0+lib.time) { /* really old shot */
			remove_shot=true; /* remove from list */
		} else { /* shot not too old */
			/* Check shots against enemies list */
			for (std::list<enemy>::iterator eit=enemys.begin();eit!=enemys.end();/* see middle */) 
			{
				enemy &e=(*eit);
				if (s.hits(e)) { /* shot hits enemy!  Remove both (after animation?) */
					remove_shot=true;
					eit=enemys.erase(eit);
				}
				else {
					++eit; /* loop increment */
				}
			}
		}
		if (remove_shot) 
		{ /* remove the current shot from the list-- kinda tricky!*/
			it=shots.erase(it);
		} else { /* not removing, so increment around loop */
			++it; /* loop increment */
		}
	}
}

int main(int argc,char *argv[]) 
{
	spritelib_run("Sprite Invaders!",800,600);
	return 0;
}
