/* "Spritelib" demo application: draw background and a few fixed objects.
  Orion Sky Lawlor, olawlor@acm.org, 2010-04-05 (Public Domain) */
#include "spritelib/spritelib.h"
#include "spritelib/spritelib.cpp"
#include <list>

class explosion {
public:
	float created; // time when explosion was created
	float x,y; // onscreen position
	explosion(float time,float x_,float y_) :created(time), x(x_), y(y_) {}
	
	bool draw(spritelib &lib) {
		static spritelib_tex glow=lib.read_tex("glow.jpg");
		float sz=400.0*(1.0-(lib.time-created));
		if (sz>0.0)
		{
			lib.draw(glow,
				x,y,
				sz,sz,
				lib.time*1000.0,
				true); // additive blending
			return true; 
		} 
		else { // remove me!
			return false;
		}
	}
};

class animal {
public:
	bool iscat;
	float x,y; // onscreen position
	animal(bool iscat_,float x_,float y_) :iscat(iscat_), x(x_), y(y_) {}
	
	void draw(spritelib &lib) {
		static spritelib_tex cat=lib.read_tex("cat.jpg");
		static spritelib_tex dog=lib.read_tex("dogmop.jpg");
		
		float vel=1000.0; // cat
		if (!iscat) vel=-100.0;
		
		y+=vel*lib.dt; // pixels per frame = pixels per second * seconds per frame
		float wid=200;
		float ht=wid;
		if (!iscat) ht*=0.5;
		lib.draw(iscat?cat:dog,
			x,y,
			wid,ht);
	}
	
	explosion * interact(animal &a,spritelib &lib) {
		if (iscat == a.iscat) return 0; // no cat-cat or dog-dog interactions!
		static spritelib_tex glow=lib.read_tex("glow.jpg");
		float dx=x-a.x, dy=y-a.y;
		float dist=sqrt(dx*dx+dy*dy);
		if (dist<50.0) {
			return new explosion(lib.time,0.5*(x+a.x),0.5*(y+a.y));
		}
		else return 0;
	}
	
};

void spritelib_draw_screen(spritelib &lib) {
/* Background */
	static spritelib_tex background=lib.read_tex("sunset.jpg");
	float ken_burns=1.1-0.1*cos(0.15*lib.time); // slowly zoom in and out
	lib.draw(background,lib.screenw/2.0,lib.screenh/2.0,
		lib.screenw*ken_burns,lib.screenh*ken_burns);
	
	
	int score=0.0;
	
/* Player character */
	static std::list<animal> animals;
	static float last_animal=0.0;
	const float animal_delay=1.0/200.0; // seconds per animal
	static std::list<explosion *> explody;
	if (lib.mouseleft && lib.time>last_animal+animal_delay) 
	{ // fire new animal
		last_animal=lib.time;
		bool iscat=rand()%2;
		if (iscat)
			animals.push_back(animal(iscat,lib.mousex,0));
		else
			animals.push_back(animal(iscat,lib.mousex,lib.screenh));
	}
	for (std::list<animal>::iterator it=animals.begin();it!=animals.end();++it) {
		animal &c=(*it);
		c.draw(lib);
		if (c.iscat && c.y>lib.screenh) {
			score+=1.0;
		}
		if ((!c.iscat) && c.y<0.0) {
			score-=1.0;
		}
		for (std::list<animal>::iterator it2=animals.begin();it2!=it;++it2)
		{
			explosion *e=c.interact((*it2),lib);
			if (e) {
				explody.push_back(e);
				it=animals.erase(it); 
				it2=animals.erase(it2);
				break;
			}
		}
	}
	
/* Explosions */
	for (std::list<explosion *>::iterator it=explody.begin();it!=explody.end();) {
		explosion *e=(*it);
		if (!e->draw(lib)) { // it's dead, Jim.
			delete e;
			it=explody.erase(it);
		}
		else
			++it;
	}
	
	std::cout<<"Day three: "<<animals.size()<<" animals, "<<score<<" score\n";
}

int main(int argc,char *argv[]) 
{
	spritelib_run("SpriteLib Demo",800,600);
	return 0;
}
