/* Spritelib demo application: "space invaders" style game.
  Orion Sky Lawlor, olawlor@acm.org, 2010-04-05 (Public Domain) */
#include "spritelib/spritelib.h"
#include "spritelib/spritelib.cpp"
#include <list>


class shot {
public:
	float created; /* time of creation (for disposal) */
	float x,y; /* current location onscreen */
	float size; /* onscreen size (pixels) */
	float angle; /* rotation angle (degrees) */
	void draw(spritelib &lib) {
		static spritelib_tex glow=lib.read_tex("glow.jpg");
		lib.draw(glow,
			x,y, // position
			size,size,
			angle,
			true); // glow!
	}
	void physics(spritelib &lib) {
		float speed=800.0; /* motion pixels/second (positive == down) */
		y+=speed*lib.dt; /* pixels/second * seconds = pixels */
		size+=100.0*lib.dt; /* size growth rate */
		angle+=1230*lib.dt; /* angular rotation rate */
	}
};


void spritelib_draw_screen(spritelib &lib) {
/* Background */
	static spritelib_tex background=lib.read_tex("sunset.jpg");
	float ken_burns=1.1-0.1*cos(0.15*lib.time); // slowly zoom in and out
	lib.draw(background,lib.screenw/2.0,lib.screenh/2.0,
		lib.screenw*ken_burns,lib.screenh*ken_burns);
	
/* Player character */
	static spritelib_tex cat=lib.read_tex("cat.jpg");
	float playerX=lib.mousex;
	float playerY=50;
	lib.draw(cat,
		playerX, // position
		playerY,
		100, // size
		100,
		0); // angle cat facing down

/* Onscreen ordinance */
	static std::list<shot> shots; /* <- static, so shots stick around between frames */
	if (lib.key[' '] || lib.mouseleft) { /* spacebar is down--consider adding shot */
		static float last_shot=0.0;
		float shot_time=1.0/15.0; /* time between shots */
		if (lib.time>last_shot+shot_time) { /* limits shot rate */
			last_shot=lib.time;
			shot s; 
			s.created=lib.time;
			s.x=playerX; s.y=playerY+40.0;
			s.size=40.0; /* onscreen size */
			s.angle=rand()%360;
			shots.push_back(s);
		}
	}
	
	/* Draw previous shots */
	for (std::list<shot>::iterator it=shots.begin();it!=shots.end();++it) 
	{
		shot &s=(*it);
		s.physics(lib);
		s.draw(lib);
		if (s.created>2.0+lib.time) shots.erase(it); /* remove from list */
	}
}

int main(int argc,char *argv[]) 
{
	spritelib_run("Sprite Invaders!",800,600);
	return 0;
}
